@@ -215,7 +215,37 @@ PyObject * meth_mix(PyObject * self, PyObject * args) {
     return NULL;
 }
 
-PyObject * meth_rotation(PyObject * self, PyObject * args) {
+PyObject * meth_cross(PyObject * self, PyObject * args) {
+    Operand a, b;
+    if (!PyArg_ParseTuple(args, "O&O&", converter, &a, converter, &b)) {
+        return NULL;
+    }
+    if (a.type == VECTOR && b.type == VECTOR) {
+        return tup(glm::cross(a.v, b.v));
+    }
+    if (a.type == QUATERNION && b.type == QUATERNION) {
+        return tup(glm::cross(a.q, b.q));
+    }
+    PyErr_Format(PyExc_TypeError, "invalid operands");
+    return NULL;
+}
+
+PyObject * meth_dot(PyObject * self, PyObject * args) {
+    Operand a, b;
+    if (!PyArg_ParseTuple(args, "O&O&", converter, &a, converter, &b)) {
+        return NULL;
+    }
+    if (a.type == VECTOR && b.type == VECTOR) {
+        return PyFloat_FromDouble(glm::dot(a.v, b.v));
+    }
+    if (a.type == QUATERNION && b.type == QUATERNION) {
+        return PyFloat_FromDouble(glm::dot(a.q, b.q));
+    }
+    PyErr_Format(PyExc_TypeError, "invalid operands");
+    return NULL;
+}
+
+PyObject * meth_rotate(PyObject * self, PyObject * args) {
     double s;
     glm::dvec3 v;
     if (!PyArg_ParseTuple(args, "d(ddd)", &s, &v.x, &v.y, &v.z)) {
@@ -224,7 +254,29 @@ PyObject * meth_rotation(PyObject * self, PyObject * args) {
     return tup(glm::angleAxis(s, v));
 }
 
-PyObject * meth_normalize(PyObject * self, PyObject * arg) {
+PyObject * meth_split(PyObject * self, PyObject * arg) {
+    Operand a;
+    if (!converter(arg, &a)) {
+        return NULL;
+    }
+    if (a.type == QUATERNION) {
+        PyObject * res = PyTuple_New(2);
+        PyTuple_SET_ITEM(res, 0, PyFloat_FromDouble(glm::angle(a.q)));
+        PyTuple_SET_ITEM(res, 1, tup(glm::axis(a.q)));
+        return res;
+    }
+    if (a.type == MATRIX) {
+        PyObject * res = PyTuple_New(3);
+        PyTuple_SET_ITEM(res, 0, tup(a.m[0]));
+        PyTuple_SET_ITEM(res, 1, tup(a.m[1]));
+        PyTuple_SET_ITEM(res, 2, tup(a.m[2]));
+        return res;
+    }
+    PyErr_Format(PyExc_TypeError, "invalid operand");
+    return NULL;
+}
+
+PyObject * meth_norm(PyObject * self, PyObject * arg) {
     Operand a;
     if (!converter(arg, &a)) {
         return NULL;
@@ -242,7 +294,7 @@ PyObject * meth_normalize(PyObject * self, PyObject * arg) {
     return NULL;
 }
 
-PyObject * meth_inverse(PyObject * self, PyObject * arg) {
+PyObject * meth_inv(PyObject * self, PyObject * arg) {
     Operand a;
     if (!converter(arg, &a)) {
         return NULL;
@@ -257,6 +309,18 @@ PyObject * meth_inverse(PyObject * self, PyObject * arg) {
     return NULL;
 }
 
+PyObject * meth_det(PyObject * self, PyObject * arg) {
+    Operand a;
+    if (!converter(arg, &a)) {
+        return NULL;
+    }
+    if (a.type == MATRIX) {
+        return PyFloat_FromDouble(glm::determinant(a.m));
+    }
+    PyErr_Format(PyExc_TypeError, "invalid operand");
+    return NULL;
+}
+
 PyObject * meth_cast(PyObject * self, PyObject * arg) {
     Operand a;
     if (!converter(arg, &a)) {
@@ -323,9 +387,13 @@ PyMethodDef module_methods[] = {
     {"sub", (PyCFunction)meth_sub, METH_VARARGS, NULL},
     {"mul", (PyCFunction)meth_mul, METH_VARARGS, NULL},
     {"mix", (PyCFunction)meth_mix, METH_VARARGS, NULL},
-    {"rotation", (PyCFunction)meth_rotation, METH_VARARGS, NULL},
-    {"normalize", (PyCFunction)meth_normalize, METH_O, NULL},
-    {"inverse", (PyCFunction)meth_inverse, METH_O, NULL},
+    {"cross", (PyCFunction)meth_cross, METH_VARARGS, NULL},
+    {"dot", (PyCFunction)meth_dot, METH_VARARGS, NULL},
+    {"rotate", (PyCFunction)meth_rotate, METH_VARARGS, NULL},
+    {"split", (PyCFunction)meth_split, METH_O, NULL},
+    {"norm", (PyCFunction)meth_norm, METH_O, NULL},
+    {"inv", (PyCFunction)meth_inv, METH_O, NULL},
+    {"det", (PyCFunction)meth_det, METH_O, NULL},
     {"cast", (PyCFunction)meth_cast, METH_O, NULL},
     {"swizzle", (PyCFunction)meth_swizzle, METH_VARARGS, NULL},
     {"pack", (PyCFunction)meth_pack, METH_O, NULL},
